class XMLTypes:
    OBJ = 'obj'
    ATT = 'att'
    VAL = 'val'
    LIST = 'list'
    ID = 'id'
    CLASS = 'class'
    UUID = 'uuid'
    DB = 'db'
    VERSION = 'version'
    FIRMWARE = 'firmware'
    NS = 'xmlns:xsi'
    LOCATION = 'xsi:noNamespaceSchemaLocation'
